<?php
/** 
*
* @package phpBB3
* @version $Id: smartfeed_url_new_key.php, v 2.2.12 Production 2009/04/23 mark@phpbbservices.com Exp $
* @copyright (c) Mark D. Hamill (mhamill@computer.org, http://phpbbservices.com) 
* @license http://opensource.org/licenses/gpl-license.php GNU Public License 
*
*/

/**
* @ignore
*/

// Written by Mark D. Hamill, mhamill@computer.org, http://phpbbservices.com
// This software is designed to work with phpBB Version 3.0.4

// This is a program that is designed to be called from an AJAX request. For a given user ID and valid
// session ID it creates a new value for user_smartfeed_key, stores it, then encrypts it and returns it 
// to the calling Javascript function in smartfeed_url_body.html. The encrypted password has two parts,
// one to be used if the IP authentication is not required, and one if it is. The two fields are delimited
// by a tilde, since one limitation of AJAX REST-like calls is that when used as a function it should 
// return only a single value.

define('IN_PHPBB', true);
$phpbb_root_path = (defined('PHPBB_ROOT_PATH')) ? PHPBB_ROOT_PATH : './';
$phpEx = substr(strrchr(__FILE__, '.'), 1);
include($phpbb_root_path . 'common.' . $phpEx);

// Start session management
$user->session_begin();
$auth->acl($user->data);

$user->setup('mods/smartfeed');
$board_url = generate_board_url() . '/';

// Check for the user_id argument
$user_id = request_var('u',0);
if ($user_id == 0)
{
	echo sprintf($user->lang['SMARTFEED_NO_U_ARGUMENT'], $phpEx);
	exit;
}

// Check for the sid argument
$sid = request_var('sid','NONE');
if ($sid == 'NONE')
{
	echo sprintf($user->lang['SMARTFEED_NO_SESSION_KEY'], $phpEx);
	exit;
}

// Validate the session key passed. If it is valid, it is assumed that no hack attempt exists so it is okay
// to generate a new smartfeed key

$sql = 'SELECT session_id
		FROM ' . SESSIONS_TABLE . '
		WHERE session_user_id = ' . intval($user_id) . " AND session_ip = '" . $user->ip . "'
		ORDER BY session_start";
$result = $db->sql_query_limit($sql, 1);
$row = $db->sql_fetchrow($result);
if ($row['session_id'] != $sid)
{
	echo sprintf($user->lang['SMARTFEED_INVALID_SESSION_KEY'], $sid, $user_id, $phpEx);
	exit;
}
$db->sql_freeresult($result);
		
// Generate a new smartfeed encryption key.
$user_smartfeed_key = gen_rand_string(32);
$encrypted_password = encrypt($user->data['user_password'], $user_smartfeed_key);
$encrypted_password_with_ip = encrypt($user->data['user_password'] . '~' . $user->ip, $user_smartfeed_key);

// Store the key
$sql = 'UPDATE ' . USERS_TABLE . "
	   SET user_smartfeed_key = '" . $db->sql_escape($user_smartfeed_key) . "'
	   WHERE user_id = " . $user_id;
$result = $db->sql_query($sql);

// Pass back the new encrypted passwords
echo $encrypted_password . '~' . $encrypted_password_with_ip; 
exit; 

function base64_encode_urlsafe($input)
{
	// Thanks to phpBB forum user klapray for this logic for creating a "urlsafe" versions of base64_encode and _decode.
	return strtr(base64_encode($input), '+/=', '-_.');
}

function encrypt($data_input, $key)
{   

	// This function encrypts $data_input with the given $key. It uses the PHP Secure Communications Library, found
	// at http://phpseclib.sourceforge.net/. If mcrypt is compiled into PHP it will use that, otherwise it will use the functions 
	// from this library. This allows for maximum portability of Smartfeed. Thanks to Terrafrost at phpbb.com for the suggestion
	// to use this library!
	
	global $phpbb_root_path, $phpEx;
	static $des;
	
	if (!isset($des))
	{
		include($phpbb_root_path . "includes/Crypt/TripleDES.$phpEx");
    	$des = new Crypt_TripleDES();
	}

	$des->setKey($key);

	// Encrypt
	$encrypted_data = base64_encode_urlsafe($des->encrypt($data_input));
	return $encrypted_data;

}

?>